==> cross_scroll-0.1.0.tar.gz: lib/src/Infinite/infinite_cross_scroll.dart <==
import 'dart:math' as math;

import 'package:cross_scroll/src/Infinite/crontroller.dart';
import 'package:cross_scroll/src/cross_scroll_decor.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';

class InfiniteCrossScroll extends StatefulWidget {
  const InfiniteCrossScroll(
      {this.controller,
      required this.maxRows,
      required this.maxColumns,
      required this.cellSize,
      required this.builder,
      this.horizontalScroll = const CrossScrollDesign(),
      this.verticalScroll = const CrossScrollDesign(),
      this.diagonalDragBehavior = DiagonalDragBehavior.free,
      this.dragStartBehavior = DragStartBehavior.start,
      this.keyboardDismissBehavior = ScrollViewKeyboardDismissBehavior.manual,
      this.primary,
      this.cacheExtent,
      super.key});
  final CrossScrollController? controller;

  final Size cellSize;
  final int? maxRows;
  final bool? primary;
  final CrossScrollDesign horizontalScroll;
  final CrossScrollDesign verticalScroll;
  final int? maxColumns;
  final Widget? Function(BuildContext, ChildVicinity) builder;
  final DiagonalDragBehavior diagonalDragBehavior;
  final DragStartBehavior dragStartBehavior;
  final ScrollViewKeyboardDismissBehavior keyboardDismissBehavior;
  final double? cacheExtent;

  @override
  State<InfiniteCrossScroll> createState() => _InfiniteCrossScrollState();
}

class _InfiniteCrossScrollState extends State<InfiniteCrossScroll> {
  ScrollController? _hController;
  ScrollController? _vController;
  late CrossScrollController primary;
  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    primary = widget.controller ?? CrossScrollController();
    _hController = ScrollController(
        initialScrollOffset: primary.initialScrollOffset.dx,
        keepScrollOffset: primary.keepScrollOffset,
        debugLabel: primary.debugLabel,
        onAttach: primary.onAttach,
        onDetach: primary.onDetach);
    _vController = ScrollController(
        initialScrollOffset: primary.initialScrollOffset.dy,
        keepScrollOffset: primary.keepScrollOffset,
        debugLabel: primary.debugLabel,
        onAttach: primary.onAttach,
        onDetach: primary.onDetach);
    primary.init(h: _hController!, v: _vController!, cellSize: widget.cellSize);
  }

  @override
  void didUpdateWidget(covariant InfiniteCrossScroll oldWidget) {
    // TODO: implement didUpdateWidget
    super.didUpdateWidget(oldWidget);
    print("DID UPDATE");
  }

  @override
  void didChangeDependencies() {
    // TODO: implement didChangeDependencies
    super.didChangeDependencies();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: TwoDimensionalGridView(
        // key: widget.key,
        controller: primary,
        primary: widget.primary,
        diagonalDragBehavior: widget.diagonalDragBehavior,
        dragStartBehavior: widget.dragStartBehavior,
        cacheExtent: widget.cacheExtent,
        keyboardDismissBehavior: widget.keyboardDismissBehavior,
        verticalDetails: ScrollableDetails.vertical(
            controller: _vController,
            reverse: widget.verticalScroll.reverse,
            physics: widget.verticalScroll.physics,
            decorationClipBehavior: widget.verticalScroll.clipBehavior),
        horizontalDetails: ScrollableDetails.horizontal(
            controller: _hController,
            reverse: widget.horizontalScroll.reverse,
            physics: widget.horizontalScroll.physics,
            decorationClipBehavior: widget.horizontalScroll.clipBehavior),
        delegate: TwoDimensionalChildBuilderDelegate(
            maxXIndex: widget.maxRows,
            maxYIndex: widget.maxColumns,
            builder: widget.builder),
      ),
    );
  }
}

class TwoDimensionalGridView extends TwoDimensionalScrollView {
  const TwoDimensionalGridView({
    super.key,
    required this.controller,
    super.primary,
    super.mainAxis = Axis.vertical,
    super.verticalDetails = const ScrollableDetails.vertical(),
    super.horizontalDetails = const ScrollableDetails.horizontal(),
    required TwoDimensionalChildBuilderDelegate delegate,
    super.cacheExtent,
    super.diagonalDragBehavior = DiagonalDragBehavior.none,
    super.dragStartBehavior = DragStartBehavior.start,
    super.keyboardDismissBehavior = ScrollViewKeyboardDismissBehavior.manual,
    super.clipBehavior = Clip.hardEdge,
  }) : super(delegate: delegate);

  final CrossScrollController controller;

  @override
  Widget buildViewport(
    BuildContext context,
    ViewportOffset verticalOffset,
    ViewportOffset horizontalOffset,
  ) {
    return TwoDimensionalGridViewport(
      controller: controller,
      horizontalOffset: horizontalOffset,
      horizontalAxisDirection: horizontalDetails.direction,
      verticalOffset: verticalOffset,
      verticalAxisDirection: verticalDetails.direction,
      mainAxis: mainAxis,
      delegate: delegate as TwoDimensionalChildBuilderDelegate,
      cacheExtent: cacheExtent,
      clipBehavior: clipBehavior,
    );
  }
}

class TwoDimensionalGridViewport extends TwoDimensionalViewport {
  const TwoDimensionalGridViewport({
    super.key,
    required this.controller,
    required super.verticalOffset,
    required super.verticalAxisDirection,
    required super.horizontalOffset,
    required super.horizontalAxisDirection,
    required TwoDimensionalChildBuilderDelegate super.delegate,
    required super.mainAxis,
    super.cacheExtent,
    super.clipBehavior = Clip.hardEdge,
  });
  final CrossScrollController controller;

  @override
  RenderTwoDimensionalViewport createRenderObject(BuildContext context) {
    return RenderTwoDimensionalGridViewport(
      controller: controller,
      horizontalOffset: horizontalOffset,
      horizontalAxisDirection: horizontalAxisDirection,
      verticalOffset: verticalOffset,
      verticalAxisDirection: verticalAxisDirection,
      mainAxis: mainAxis,
      delegate: delegate as TwoDimensionalChildBuilderDelegate,
      childManager: context as TwoDimensionalChildManager,
      cacheExtent: cacheExtent,
      clipBehavior: clipBehavior,
    );
  }

  @override
  void updateRenderObject(
    BuildContext context,
    RenderTwoDimensionalGridViewport renderObject,
  ) {
    renderObject
      ..horizontalOffset = horizontalOffset
      ..horizontalAxisDirection = horizontalAxisDirection
      ..verticalOffset = verticalOffset
      ..verticalAxisDirection = verticalAxisDirection
      ..mainAxis = mainAxis
      ..delegate = delegate
      ..cacheExtent = cacheExtent
      ..clipBehavior = clipBehavior;
  }
}

class RenderTwoDimensionalGridViewport extends RenderTwoDimensionalViewport {
  RenderTwoDimensionalGridViewport({
    required this.controller,
    required super.horizontalOffset,
    required super.horizontalAxisDirection,
    required super.verticalOffset,
    required super.verticalAxisDirection,
    required TwoDimensionalChildBuilderDelegate delegate,
    required super.mainAxis,
    required super.childManager,
    super.cacheExtent,
    super.clipBehavior = Clip.hardEdge,
  }) : super(delegate: delegate);
  final CrossScrollController controller;

  @override
  void layoutChildSequence() {
    final double horizontalPixels = horizontalOffset.pixels;
    final double verticalPixels = verticalOffset.pixels;
    final double viewportWidth = viewportDimension.width + cacheExtent;
    final double viewportHeight = viewportDimension.height + cacheExtent;
    final TwoDimensionalChildBuilderDelegate builderDelegate =
        delegate as TwoDimensionalChildBuilderDelegate;
    Size tileSize = controller.size;

    final int maxRowIndex = builderDelegate.maxYIndex!;
    final int maxColumnIndex = builderDelegate.maxXIndex!;

    final int leadingColumn =
        math.max((horizontalPixels / tileSize.width).floor(), 0);
    final int leadingRow =
        math.max((verticalPixels / tileSize.height).floor(), 0);
    final int trailingColumn = math.min(
      ((horizontalPixels + viewportWidth) / tileSize.width).ceil(),
      maxColumnIndex,
    );
    final int trailingRow = math.min(
      ((verticalPixels + viewportHeight) / tileSize.height).ceil(),
      maxRowIndex,
    );

    double xLayoutOffset =
        (leadingColumn * tileSize.width) - horizontalOffset.pixels;
    for (int column = leadingColumn; column <= trailingColumn; column++) {
      double yLayoutOffset =
          (leadingRow * tileSize.height) - verticalOffset.pixels;
      for (int row = leadingRow; row <= trailingRow; row++) {
        final ChildVicinity vicinity =
            ChildVicinity(xIndex: column, yIndex: row);
        final RenderBox child = buildOrObtainChildFor(vicinity)!;
        child.layout(constraints.loosen());

        // Subclasses only need to set the normalized layout offset. The super
        // class adjusts for reversed axes.
        parentDataOf(child).layoutOffset = Offset(xLayoutOffset, yLayoutOffset);
        yLayoutOffset += tileSize.height;
      }
      xLayoutOffset += tileSize.width;
    }

    // Set the min and max scroll extents for each axis.
    final double verticalExtent = tileSize.height * (maxRowIndex + 1);
    verticalOffset.applyContentDimensions(
      0.0,
      clampDouble(
          verticalExtent - viewportDimension.height, 0.0, double.infinity),
    );
    final double horizontalExtent = tileSize.width * (maxColumnIndex + 1);
    horizontalOffset.applyContentDimensions(
      0.0,
      clampDouble(
          horizontalExtent - viewportDimension.width, 0.0, double.infinity),
    );
    // Super class handles garbage collection too!
  }
}

==> cross_scroll-0.1.1.tar.gz: lib/src/Infinite/infinite_cross_scroll.dart <==
import 'dart:math' as math;

import 'package:cross_scroll/src/Infinite/crontroller.dart';
import 'package:cross_scroll/src/cross_scroll_decor.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';

/// [InfiniteCrossScroll] is a   Widget that provides an infinite two-dimensional scrolling grid including diagonal scrolling.
///
/// It extends the TwoDimensionalScrollView, allowing for both horizontal and vertical scrolling with an
/// arbitrary number of rows and columns. The grid cells are represented by the provided builder function,
/// and the entire widget is backed by a CrossScrollController for efficient scroll control.
///
/// ## Parameters:
/// - `controller`: An optional CrossScrollController that can be provided to control the scrolling behavior.
///   If not provided, a new controller will be created internally.
/// - `cellSize`: The size of each grid cell in the two-dimensional grid.
/// - `maxRows`: The maximum number of rows in the grid.
/// - `maxColumns`: The maximum number of columns in the grid.
/// - `builder`: A function that builds the content of each grid cell based on its location.
/// - `horizontalScroll`: Configuration for horizontal scrolling behavior, with optional parameters like reverse and physics.
/// - `verticalScroll`: Configuration for vertical scrolling behavior, with optional parameters like reverse and physics.
/// - `diagonalDragBehavior`: Defines the diagonal drag behavior, with options like 'free', 'constrained', or 'disabled'.
/// - `dragStartBehavior`: Determines when the drag behavior should start, with options like 'start' or 'down'.
/// - `keyboardDismissBehavior`: Defines the keyboard dismiss behavior for the ScrollView, with options like 'manual' or 'onDrag'.
/// - `primary`: Indicates whether this ScrollView is the primary scroll view.
/// - `cacheExtent`: The maximum number of pixels away from the currently visible pixels that the framework should build children.
///
/// ## Scroll bars
/// To add scrollbars to [InfiniteCrossScroll]  just wrap it in [CrossScrollBar] and provide the same controller to both,
///
///
/// ## Usage Example:
/// ```dart
/// InfiniteCrossScroll(
///   cellSize: Size(100, 100),
///   maxRows: 20,
///   maxColumns: 20,
///   builder: (BuildContext context, ChildVicinity vicinity) {
///     // Build and return the content for each grid cell based on its position.
///     return MyGridCellContent(vicinity);
///   },
/// )
/// ```
class InfiniteCrossScroll extends StatefulWidget {
  const InfiniteCrossScroll(
      {this.controller,
      required this.maxRows,
      required this.maxColumns,
      required this.cellSize,
      required this.builder,
      this.horizontalScroll = const CrossScrollDesign(),
      this.verticalScroll = const CrossScrollDesign(),
      this.diagonalDragBehavior = DiagonalDragBehavior.free,
      this.dragStartBehavior = DragStartBehavior.start,
      this.keyboardDismissBehavior = ScrollViewKeyboardDismissBehavior.manual,
      this.primary,
      this.cacheExtent,
      super.key});
  final CrossScrollController? controller;

  final Size cellSize;
  final int? maxRows;
  final bool? primary;
  final CrossScrollDesign horizontalScroll;
  final CrossScrollDesign verticalScroll;
  final int? maxColumns;
  final Widget? Function(BuildContext, ChildVicinity) builder;
  final DiagonalDragBehavior diagonalDragBehavior;
  final DragStartBehavior dragStartBehavior;
  final ScrollViewKeyboardDismissBehavior keyboardDismissBehavior;
  final double? cacheExtent;

  @override
  State<InfiniteCrossScroll> createState() => _InfiniteCrossScrollState();
}

class _InfiniteCrossScrollState extends State<InfiniteCrossScroll> {
  ScrollController? _hController;
  ScrollController? _vController;
  late CrossScrollController primary;
  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    primary = widget.controller ?? CrossScrollController();
    _hController = ScrollController(
        initialScrollOffset: primary.initialScrollOffset.dx,
        keepScrollOffset: primary.keepScrollOffset,
        debugLabel: primary.debugLabel,
        onAttach: primary.onAttach,
        onDetach: primary.onDetach);
    _vController = ScrollController(
        initialScrollOffset: primary.initialScrollOffset.dy,
        keepScrollOffset: primary.keepScrollOffset,
        debugLabel: primary.debugLabel,
        onAttach: primary.onAttach,
        onDetach: primary.onDetach);
    primary.init(h: _hController!, v: _vController!, cellSize: widget.cellSize);
  }

  @override
  void didUpdateWidget(covariant InfiniteCrossScroll oldWidget) {
    // TODO: implement didUpdateWidget
    super.didUpdateWidget(oldWidget);
    print("DID UPDATE");
  }

  @override
  void didChangeDependencies() {
    // TODO: implement didChangeDependencies
    super.didChangeDependencies();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: TwoDGridView(
        // key: widget.key,
        controller: primary,
        primary: widget.primary,
        diagonalDragBehavior: widget.diagonalDragBehavior,
        dragStartBehavior: widget.dragStartBehavior,
        cacheExtent: widget.cacheExtent,
        keyboardDismissBehavior: widget.keyboardDismissBehavior,
        verticalDetails: ScrollableDetails.vertical(
            controller: _vController,
            reverse: widget.verticalScroll.reverse,
            physics: widget.verticalScroll.physics,
            decorationClipBehavior: widget.verticalScroll.clipBehavior),
        horizontalDetails: ScrollableDetails.horizontal(
            controller: _hController,
            reverse: widget.horizontalScroll.reverse,
            physics: widget.horizontalScroll.physics,
            decorationClipBehavior: widget.horizontalScroll.clipBehavior),
        delegate: TwoDimensionalChildBuilderDelegate(
            maxXIndex: widget.maxRows,
            maxYIndex: widget.maxColumns,
            builder: widget.builder),
      ),
    );
  }
}

class TwoDGridView extends TwoDimensionalScrollView {
  const TwoDGridView({
    super.key,
    required this.controller,
    super.primary,
    super.mainAxis = Axis.vertical,
    super.verticalDetails = const ScrollableDetails.vertical(),
    super.horizontalDetails = const ScrollableDetails.horizontal(),
    required TwoDimensionalChildBuilderDelegate delegate,
    super.cacheExtent,
    super.diagonalDragBehavior = DiagonalDragBehavior.none,
    super.dragStartBehavior = DragStartBehavior.start,
    super.keyboardDismissBehavior = ScrollViewKeyboardDismissBehavior.manual,
    super.clipBehavior = Clip.hardEdge,
  }) : super(delegate: delegate);

  final CrossScrollController controller;

  @override
  Widget buildViewport(
    BuildContext context,
    ViewportOffset verticalOffset,
    ViewportOffset horizontalOffset,
  ) {
    return TwoDGridViewport(
      controller: controller,
      horizontalOffset: horizontalOffset,
      horizontalAxisDirection: horizontalDetails.direction,
      verticalOffset: verticalOffset,
      verticalAxisDirection: verticalDetails.direction,
      mainAxis: mainAxis,
      delegate: delegate as TwoDimensionalChildBuilderDelegate,
      cacheExtent: cacheExtent,
      clipBehavior: clipBehavior,
    );
  }
}

class TwoDGridViewport extends TwoDimensionalViewport {
  const TwoDGridViewport({
    super.key,
    required this.controller,
    required super.verticalOffset,
    required super.verticalAxisDirection,
    required super.horizontalOffset,
    required super.horizontalAxisDirection,
    required TwoDimensionalChildBuilderDelegate super.delegate,
    required super.mainAxis,
    super.cacheExtent,
    super.clipBehavior = Clip.hardEdge,
  });
  final CrossScrollController controller;

  @override
  RenderTwoDimensionalViewport createRenderObject(BuildContext context) {
    return RenderTwoDimensionalGridViewport(
      controller: controller,
      horizontalOffset: horizontalOffset,
      horizontalAxisDirection: horizontalAxisDirection,
      verticalOffset: verticalOffset,
      verticalAxisDirection: verticalAxisDirection,
      mainAxis: mainAxis,
      delegate: delegate as TwoDimensionalChildBuilderDelegate,
      childManager: context as TwoDimensionalChildManager,
      cacheExtent: cacheExtent,
      clipBehavior: clipBehavior,
    );
  }

  @override
  void updateRenderObject(
    BuildContext context,
    RenderTwoDimensionalGridViewport renderObject,
  ) {
    renderObject
      ..horizontalOffset = horizontalOffset
      ..horizontalAxisDirection = horizontalAxisDirection
      ..verticalOffset = verticalOffset
      ..verticalAxisDirection = verticalAxisDirection
      ..mainAxis = mainAxis
      ..delegate = delegate
      ..cacheExtent = cacheExtent
      ..clipBehavior = clipBehavior;
  }
}

class RenderTwoDimensionalGridViewport extends RenderTwoDimensionalViewport {
  RenderTwoDimensionalGridViewport({
    required this.controller,
    required super.horizontalOffset,
    required super.horizontalAxisDirection,
    required super.verticalOffset,
    required super.verticalAxisDirection,
    required TwoDimensionalChildBuilderDelegate delegate,
    required super.mainAxis,
    required super.childManager,
    super.cacheExtent,
    super.clipBehavior = Clip.hardEdge,
  }) : super(delegate: delegate);
  final CrossScrollController controller;

  @override
  void layoutChildSequence() {
    final double horizontalPixels = horizontalOffset.pixels;
    final double verticalPixels = verticalOffset.pixels;
    final double viewportWidth = viewportDimension.width + cacheExtent;
    final double viewportHeight = viewportDimension.height + cacheExtent;
    final TwoDimensionalChildBuilderDelegate builderDelegate =
        delegate as TwoDimensionalChildBuilderDelegate;
    Size tileSize = controller.size;

    final int maxRowIndex = builderDelegate.maxYIndex!;
    final int maxColumnIndex = builderDelegate.maxXIndex!;

    final int leadingColumn =
        math.max((horizontalPixels / tileSize.width).floor(), 0);
    final int leadingRow =
        math.max((verticalPixels / tileSize.height).floor(), 0);
    final int trailingColumn = math.min(
      ((horizontalPixels + viewportWidth) / tileSize.width).ceil(),
      maxColumnIndex,
    );
    final int trailingRow = math.min(
      ((verticalPixels + viewportHeight) / tileSize.height).ceil(),
      maxRowIndex,
    );

    double xLayoutOffset =
        (leadingColumn * tileSize.width) - horizontalOffset.pixels;
    for (int column = leadingColumn; column <= trailingColumn; column++) {
      double yLayoutOffset =
          (leadingRow * tileSize.height) - verticalOffset.pixels;
      for (int row = leadingRow; row <= trailingRow; row++) {
        final ChildVicinity vicinity =
            ChildVicinity(xIndex: column, yIndex: row);
        final RenderBox child = buildOrObtainChildFor(vicinity)!;
        child.layout(constraints.loosen());

        // Subclasses only need to set the normalized layout offset. The super
        // class adjusts for reversed axes.
        parentDataOf(child).layoutOffset = Offset(xLayoutOffset, yLayoutOffset);
        yLayoutOffset += tileSize.height;
      }
      xLayoutOffset += tileSize.width;
    }

    // Set the min and max scroll extents for each axis.
    final double verticalExtent = tileSize.height * (maxRowIndex + 1);
    verticalOffset.applyContentDimensions(
      0.0,
      clampDouble(
          verticalExtent - viewportDimension.height, 0.0, double.infinity),
    );
    final double horizontalExtent = tileSize.width * (maxColumnIndex + 1);
    horizontalOffset.applyContentDimensions(
      0.0,
      clampDouble(
          horizontalExtent - viewportDimension.width, 0.0, double.infinity),
    );
    // Super class handles garbage collection too!
  }
}
